/*
 * sverb: an order 15 CFDN reverb
 * http://sed.free.fr/sverb
 * public domain
 * (there may be some patent for this stuff, I don't know and I don't care)
 * Cdric Roux <sed@free.fr>
 *
 * Algorithm comes from:
 * "Maximally Diffusive Yet Efficient Feedback Delay Networks for
 *  Artificial Reverberation", Davide Rocchesso, IEEE SIGNAL
 * PROCESSING LETTERS, VOL. 4, NO. 9, SEPTEMBER 1997
 * http://www.scienze.univr.it/documenti//ArticoloRivista/allegato/
 *   allegato128605.pdf
 * And:
 * "Circulant and elliptic feedback delay networks for artificial
 *  reverberation", D. Rocchesso and J. O. Smith, IEEE Trans. Speech
 * Audio Processing, vol. 5, pp. 51 63, Jan. 1997.
 * http://ccrma.stanford.edu/~jos/cfdn/cfdn.pdf
 * And:
 * http://ccrma.stanford.edu/~jos/Reverb/First_Order_Delay_Filter_Design.html
 * http://ccrma.stanford.edu/~jos/Reverb/Tonal_Correction_Filter.html
 * for the lowpass filters
 */

/* take the help.txt file and produce help_gui.h */
/* could be done in sed or anything else I guess... */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int size;
int maxsize;
char *buf;

void put_char(int c)
{
  if (size == maxsize) {
    buf = realloc(buf, maxsize += 1024);
    if (!buf) exit(1);
  }
  buf[size++] = c;
}

void ultra_low_handle(char *type, char *s, int bold)
{
  printf("  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter,\n");
  printf("    \"%s\",\n", s);
  if (bold)
    printf("    -1, \"%s\", \"b\", 0);\n", type);
  else
    printf("    -1, \"%s\", 0);\n", type);
}

void low_handle(char *type, char *s, int bold)
{
  int len;

  /* we split long strings, for the compiler to feel cool */
  while ((len=strlen(s)) > 70) {
    int c = s[70];
    s[70] = 0;
    ultra_low_handle(type, s, bold);
    s[70] = c;
    s += 70;
  }

  ultra_low_handle(type, s, bold);
}

void handle_it(char *type, char *s, int bold)
{
  char *sep;

#if 0
  while (s[0] && (s[0] == ' ' || s[0] == '\n')) s++;
  if (!s[0]) return;
#endif

  while (1) {
    sep = strstr(s, ".b ");
    if (!sep)
      return low_handle(type, s, bold);

    *sep = 0;
    low_handle(type, s, bold);

    s = sep + 3;
    bold = 1 - bold;
  }
}

void handle_line(void)
{
  if (!size) return;
  while (size && (buf[size-1] == ' ' || buf[size-1] == '\n')) size--;
  if (!size) return;
  put_char('\\');
  put_char('n');
  put_char(0);

  if (!strncmp(buf, ".p ", 3)) {
    size -= 3;
    return handle_it("p", buf+3, 0);
  }
  if (!strncmp(buf, ".h1 ", 4)) {
    size -= 4;
    return handle_it("h1", buf+4, 0);
  }
  if (!strncmp(buf, ".h2 ", 4)) {
    size -= 4;
    return handle_it("h2", buf+4, 0);
  }
  if (!strncmp(buf, ".h3 ", 4)) {
    size -= 4;
    return handle_it("h3", buf+4, 0);
  }
  handle_it("p", buf, 0);
}

int main(void)
{
  int last_eol = 0;
  int last_space = 0;

  while (1) {
    int c = getchar();
    if (c == -1) break;
    if (c == '.' && last_eol) {
      handle_line();
      size = 0;
    }
    if (!((c == '\n' || c == ' ') && last_space == 1)) {
      int d = c;
      if (d == '\\') put_char('\\');
      if (d == '\"') put_char('\\');
      if (d == '\n') d = ' ';
      put_char(d);
    }
    if (c == '\n') last_eol = 1;
    else last_eol = 0;
#if 0
    if (c == '\n' || c == ' ') last_space = 1;
    else last_space = 0;
#endif
  }

  if (size)
    handle_line();

  return 0;
}
